import pytest

from nbstore.store import Store


@pytest.fixture(scope="module", autouse=True)
def _execute(store: Store):
    return store.execute("text.ipynb")


def test_mime_content_stdout(store: Store):
    content = store.get_mime_content("text.ipynb", "text:stdout")
    assert isinstance(content, tuple)
    assert content[0] == "text/plain"
    assert content[1] == "'stdout'"


def test_mime_content_stream(store: Store):
    content = store.get_mime_content("text.ipynb", "text:stream")
    assert isinstance(content, tuple)
    assert content[0] == "text/plain"
    assert content[1] == "stream1\nstream2\n"


def test_mime_content_both(store: Store):
    content = store.get_mime_content("text.ipynb", "text:both")
    assert isinstance(content, tuple)
    assert content[0] == "text/plain"
    assert content[1] == "'hello'"


def test_mime_content_pandas(store: Store):
    content = store.get_mime_content("text.ipynb", "text:pandas")
    assert isinstance(content, tuple)
    assert content[0] == "text/html"
    assert isinstance(content[1], str)
    assert content[1].startswith("<div>")


def test_mime_content_polars(store: Store):
    content = store.get_mime_content("text.ipynb", "text:polars")
    assert isinstance(content, tuple)
    assert content[0] == "text/html"
    assert isinstance(content[1], str)
    assert content[1].startswith("<div>")



import re
from pathlib import Path

import pytest
from PIL import Image

from nbstore.store import Store


@pytest.fixture(scope="module")
def data(store: Store):
    return store.get_data("raster.ipynb", "fig:raster")


def test_data(data):
    assert isinstance(data, dict)
    assert len(data) == 2
    assert "text/plain" in data


@pytest.fixture(scope="module")
def text(data: dict[str, str]):
    return data["text/plain"]


def test_backend(text: str):
    assert text.startswith("%% Creator: Matplotlib, PGF backend")


def test_convert(text: str):
    i = 0

    for k, filename in enumerate(
        re.findall(r"\{\\includegraphics\[.+?\]\{(.+?)\}\}", text),
    ):
        assert isinstance(filename, str)
        assert filename.endswith(".png")
        assert Path(filename).exists()
        image = Image.open(filename)
        assert image.format == "PNG"
        if k == 0:
            assert image.size == (71, 71)
        else:
            assert image.size == (141, 141)
        i += 1

    assert i == 2



from __future__ import annotations

import base64
import io
import tempfile
import uuid
from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Callable

    from IPython.lib.pretty import RepresentationPrinter
    from matplotlib.figure import Figure
    from seaborn.objects import Plot


def matplotlib_figure_to_pgf(fig: Figure, rp: RepresentationPrinter, cycle) -> None:
    name = str(uuid.uuid4())

    with tempfile.TemporaryDirectory() as dirname:
        directory = Path(dirname)
        filename = f"{name}.pgf"
        path = directory / filename
        fig.savefig(path, format="pgf", bbox_inches="tight")
        text = path.read_text(encoding="utf-8")

        imagenames = [x.name for x in directory.iterdir() if x.name != filename]
        if not imagenames:
            return rp.text(text)

        for name in imagenames:
            text = text.replace(name, _encode_pgf_text(name, directory))

        return rp.text(text)


def _encode_pgf_text(name: str, directory: Path) -> str:
    ext = name.rsplit(".", 1)[-1]
    data = (directory / name).read_bytes()
    b64 = base64.b64encode(data).decode()
    return f"data:image/{ext};base64,{b64}"


def matplotlib_figure_to_pdf(fig: Figure) -> bytes:
    with io.BytesIO() as fp:
        fig.savefig(fp, format="pdf", bbox_inches="tight")
        return fp.getvalue()


def matplotlib_figure_to_svg(fig: Figure) -> str:
    with io.StringIO() as fp:
        fig.savefig(fp, format="svg", bbox_inches="tight")
        return fp.getvalue()


def seaborn_plot_to_pgf(plot: Plot, rp: RepresentationPrinter, cycle) -> None:
    from seaborn._core.plot import theme_context

    plotter = plot.plot()
    with theme_context(plotter._theme):  # type: ignore
        return matplotlib_figure_to_pgf(plotter._figure, rp, cycle)  # type: ignore


def seaborn_plot_to_pdf(plot: Plot) -> bytes:
    from seaborn._core.plot import theme_context

    plotter = plot.plot()
    with theme_context(plotter._theme):  # type: ignore
        return matplotlib_figure_to_pdf(plotter._figure)  # type: ignore


def seaborn_plot_to_svg(plot: Plot) -> str:
    from seaborn._core.plot import theme_context

    plotter = plot.plot()
    with theme_context(plotter._theme):  # type: ignore
        return matplotlib_figure_to_svg(plotter._figure)  # type: ignore


MIMES: dict[str, str] = {
    "pgf": "text/plain",
    "pdf": "application/pdf",
    "svg": "image/svg+xml",
}

MODULE_CLASSES: dict[str, list[tuple[str, str]]] = {
    "matplotlib": [("matplotlib.figure", "Figure")],
    "seaborn": [("seaborn._core.plot", "Plot")],
}

FUNCTIONS: dict[tuple[str, str], dict[str, Callable]] = {
    ("matplotlib.figure", "Figure"): {
        "pgf": matplotlib_figure_to_pgf,
        "pdf": matplotlib_figure_to_pdf,
        "svg": matplotlib_figure_to_svg,
    },
    ("seaborn._core.plot", "Plot"): {
        "pgf": seaborn_plot_to_pgf,
        "pdf": seaborn_plot_to_pdf,
        "svg": seaborn_plot_to_svg,
    },
}


try:
    from holoviews.ipython.display_hooks import display_hook, image_display

    @display_hook
    def pgf_display(element, max_frames):
        """Used to render elements to PGF if requested in the display formats."""
        return image_display(element, max_frames, fmt="pgf")

except ModuleNotFoundError:  # no cov
    pgf_display = None  # type: ignore


def set_formatter_holoviews(fmt: str) -> None:
    from holoviews.core.dimension import LabelledData
    from holoviews.core.options import Store

    if fmt not in Store.display_formats:
        Store.display_formats.append(fmt)

    if fmt == "pgf" and pgf_display:
        Store.set_display_hook(fmt, LabelledData, pgf_display)


def set_formatter(module: str, fmt: str, ip=None) -> None:
    if module == "holoviews":
        set_formatter_holoviews(fmt)
        return

    try:
        from IPython.core.getipython import get_ipython
    except ModuleNotFoundError:  # no cov
        msg = "IPython is not installed"
        raise ModuleNotFoundError(msg) from None

    if not ip and not (ip := get_ipython()):
        return

    if not (mime := MIMES.get(fmt)):
        raise NotImplementedError

    formatter = ip.display_formatter.formatters[mime]  # type:ignore

    if module_classes := MODULE_CLASSES.get(module):
        for module_class in module_classes:
            if function := FUNCTIONS.get(module_class, {}).get(fmt):
                formatter.for_type_by_name(*module_class, function)



import pytest

from nbstore.store import Store


@pytest.fixture(scope="module", autouse=True)
def _execute(store: Store):
    return store.execute("html.ipynb")


def test_outputs(store: Store):
    outputs = store.get_outputs("html.ipynb", "html:text")
    assert isinstance(outputs, list)
    assert len(outputs) == 1
    assert isinstance(outputs[0], dict)
    assert outputs[0]["output_type"] == "execute_result"
    assert "text/html" in outputs[0]["data"]


def test_data(store: Store):
    data = store.get_data("html.ipynb", "html:text")
    assert isinstance(data, dict)
    assert len(data) == 2
    assert "text/html" in data
    assert data["text/html"] == "<p><strong>Hello, World!</strong></p>"


def test_mime_content(store: Store):
    content = store.get_mime_content("html.ipynb", "html:text")
    assert isinstance(content, tuple)
    assert len(content) == 2
    assert isinstance(content[1], str)
    assert content[0] == "text/html"
    assert content[1] == "<p><strong>Hello, World!</strong></p>"


def test_mime_content_png(store: Store):
    content = store.get_mime_content("html.ipynb", "html:png")
    assert isinstance(content, tuple)
    assert len(content) == 2
    assert isinstance(content[1], str)
    assert content[0] == "text/html"
    assert content[1].startswith("<img src='data:image/png;base64,iVBOR")



import pytest

from nbstore.store import Store


def test_add_data(store: Store):
    from nbstore.store import get_data_by_type

    url = "add.ipynb"
    identifier = "fig:add"
    mime = "mime"
    data = "data"

    assert mime not in store.get_data(url, identifier)

    store.add_data(url, identifier, mime, data)

    assert mime in store.get_data(url, identifier)
    store.save_notebook(url)

    assert mime in store.get_data(url, identifier)

    outputs = store.get_outputs(url, identifier)
    output = get_data_by_type(outputs, "display_data")
    assert output
    store.delete_data(url, identifier, mime)
    store.save_notebook(url)

    assert mime not in store.get_data(url, identifier)


def test_get_language(store: Store):
    assert store.get_language("add.ipynb") == "python"


def test_get_data_empty(store: Store):
    assert store.get_data("pgf.ipynb", "fig:none") == {}


def test_get_data_error(store: Store):
    with pytest.raises(ValueError, match="Unknown identifier: fig:invalid"):
        store.get_data("pgf.ipynb", "fig:invalid")


def test_current_path(store: Store):
    a = store.get_source("pgf.ipynb", "fig:stream")
    b = store.get_source("", "fig:stream")
    assert a == b


def test_current_path_include_identifier(store: Store):
    a = store.get_source("pgf.ipynb", "fig:stream", include_identifier=True)
    assert a == "# #fig:stream\nprint(123)"


def test_abs_path_error(store: Store):
    with pytest.raises(ValueError, match="Notebook not found"):
        store.get_abs_path("unknown.ipynb")


def test_abs_path_error_with_current_path(store: Store):
    store.current_path = None
    with pytest.raises(ValueError, match="No active notebook."):
        store.get_notebook("")


def test_stream_none():
    from nbstore.store import get_stream

    assert get_stream([]) is None



from nbstore.store import Store


def test_cell(store: Store):
    cell = store.get_cell("svg.ipynb", "fig:svg")
    assert isinstance(cell, dict)
    assert "cell_type" in cell


def test_source(store: Store):
    source = store.get_source("svg.ipynb", "fig:svg")
    assert isinstance(source, str)
    assert "plot" in source


def test_outputs(store: Store):
    outputs = store.get_outputs("svg.ipynb", "fig:svg")
    assert isinstance(outputs, list)
    assert len(outputs) == 2
    assert isinstance(outputs[0], dict)
    assert outputs[0]["output_type"] == "execute_result"
    assert "text/plain" in outputs[0]["data"]
    assert isinstance(outputs[1], dict)
    assert outputs[1]["output_type"] == "display_data"


def test_data(store: Store):
    data = store.get_data("svg.ipynb", "fig:svg")
    assert isinstance(data, dict)
    assert len(data) == 3
    assert "text/plain" in data
    assert "image/png" in data
    assert data["image/svg+xml"].startswith('<?xml version="1.0"')


def test_mime_content(store: Store):
    mime_content = store.get_mime_content("svg.ipynb", "fig:svg")
    assert isinstance(mime_content, tuple)
    mime, content = mime_content
    assert mime == "image/svg+xml"
    assert isinstance(content, str)
    assert content.startswith('<?xml version="1.0"')



from __future__ import annotations

import base64


def get_mime_content(data: dict[str, str]) -> tuple[str, str | bytes] | None:
    """Get the content of a notebook cell.

    Args:
        data (dict[str, str]): The data of a notebook cell.

    Returns:
        tuple[str, str | bytes] | None: A tuple of the mime type and the content.
    """
    for mime in ["image/svg+xml", "text/html"]:
        if text := data.get(mime):
            return mime, text

    if text := data.get("application/pdf"):
        return "application/pdf", base64.b64decode(text)

    for mime, text in data.items():
        if mime.startswith("image/"):
            return mime, base64.b64decode(text)

    if "text/plain" in data:
        return "text/plain", data["text/plain"]

    return None



import base64

import pytest

from nbstore.store import Store


@pytest.fixture(scope="module", params=["holoviews", "hvplot"])
def lib(request: pytest.FixtureRequest):
    return request.param


@pytest.fixture(scope="module")
def nb(store: Store, lib: str):
    return store.execute(f"{lib}.ipynb")


@pytest.fixture(scope="module")
def data(store: Store, lib: str, nb):
    return store.get_data(f"{lib}.ipynb", f"fig:{lib}")


def test_type(data: dict):
    assert isinstance(data, dict)


def test_len(data: dict):
    assert len(data) == 3


@pytest.mark.parametrize("mime", ["text/html", "text/pgf", "text/plain"])
def test_mime(data: dict, mime):
    assert mime in data


def test_data_decode(data: dict):
    text = data["text/pgf"]
    assert isinstance(text, str)
    text = base64.b64decode(text).decode(encoding="utf-8")
    assert text.startswith("%% Creator: Matplotlib,")



from __future__ import annotations

from pathlib import Path

import pytest

from nbstore.store import Store


@pytest.fixture(scope="session")
def notebook_dir() -> Path:
    return Path(__file__).parent / "notebooks"


@pytest.fixture(scope="session")
def store(notebook_dir):
    return Store(notebook_dir)



from __future__ import annotations

import atexit
import base64
import re
import tempfile
from pathlib import Path

BASE64_PATTERN = re.compile(r"\{data:image/(?P<ext>.*?);base64,(?P<b64>.*?)\}")


def convert(text: str) -> str:
    return BASE64_PATTERN.sub(replace, text)


def replace(match: re.Match) -> str:
    ext = match.group("ext")
    data = base64.b64decode(match.group("b64"))

    with tempfile.NamedTemporaryFile(suffix=f".{ext}", delete=False) as tmp:
        tmp.write(data)
        path = Path(tmp.name)

    atexit.register(lambda p=path: p.unlink(missing_ok=True))

    return f"{{{path.absolute()}}}"
